package com.yem.em.task;

import com.yem.em.utils.AtlasUtils;
import com.yem.wm.utils.YEM;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.exception.KDException;
import kd.bos.schedule.executor.AbstractTask;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

import static com.yem.em.task.MatchingPLMChangeUpdateTaskEdit.*;
import static com.yem.em.utils.AtlasUtils.seelctDyn;

/**
 * @Description: 1、如《PLM变更单》分录【替换关系】=【相互替换】【新替换旧】【互不替换】,分录【启用停用状态】=‘停用’，并且【替换件启用停用状态】=‘启用’，更新当前行【执行状态】为【完成】、【完成时间】默认当前时间，并生成替换关系，记录替换关系id以及行id
 * <p>
 * 2、如《PLM变更单》分录【替换关系】=【新增】,分录【替换件启用停用状态】=‘启用’，更新当前行【执行状态】为【完成】、【完成时间】默认当前时间，并生成替换关系，记录替换关系id以及行id
 * <p>
 * 3、如《PLM变更单》分录【替换关系】=【删除】,分录【启用停用状态】=‘停用’，更新当前行【执行状态】为【完成】、【完成时间】默认当前时间，并生成替换关系，记录替换关系id以及行id
 * @Date: 2024/8/29 15:38
 * @Created: by ZZSLL
 */

public class GenerateReplaceRelationshipTaskEdit extends AbstractTask {

    private final static Logger logger = LoggerFactory.getLogger(GenerateReplaceRelationshipTaskEdit.class);

    @Override
    public void execute(RequestContext requestContext, Map<String, Object> map) throws KDException {

        DynamicObject[] plms = queryPLMChange();

        for (DynamicObject plmObj : plms) {
            DynamicObjectCollection collection = plmObj.getDynamicObjectCollection("yem_change_detail");
            DynamicObjectCollection situation_replace = new DynamicObjectCollection();
            DynamicObjectCollection situation_add = new DynamicObjectCollection();
            DynamicObjectCollection situation_delete = new DynamicObjectCollection();

            for (DynamicObject plm : collection) {
                String enablesta = plm.getString("yem_enablesta");//替换件停用启用状态
                String enablestatus_before = plm.getString("yem_enablestatus_before");//停用启用状态
                String invokstatus = plm.getString("yem_invokstatus");
                String exchangetype = plm.getString("yem_exchangetype");

                //【替换关系】=【相互替换】【新替换旧】【互不替换】 && 【启用停用状态】=‘停用’ && 【替换件启用停用状态】=‘启用’ && 执行状态 != 已完成
                if ("A-B-C".contains(exchangetype) && "A".equals(enablesta) && "B".equals(enablestatus_before) && !"B".equals(invokstatus)) {
                    situation_replace.add(plm);
                }

                // 【替换关系】=【新增】 && 【替换件启用停用状态】=‘启用’
                if ("D".equals(exchangetype) && "A".equals(enablesta) && !"B".equals(invokstatus)) {
                    situation_add.add(plm);
                }
                // 【替换关系】=【删除】 && 分录【启用停用状态】=‘停用’
                if ("E".equals(exchangetype) && "B".equals(enablestatus_before) && !"B".equals(invokstatus)) {
                    situation_delete.add(plm);
                }
            }
            generateReplaceRelation(plmObj, situation_replace);
            generateReplaceRelation(plmObj, situation_add);
            generateReplaceRelation(plmObj, situation_delete);
        }
    }

    /**
     * 生成替换关系，更新完成时间、状态
     *
     * @param plmObj
     * @param enabled
     */
    private void generateReplaceRelation(DynamicObject plmObj, DynamicObjectCollection enabled) {
        DynamicObjectCollection collection = plmObj.getDynamicObjectCollection("yem_change_detail");
        // 更新是否资源、是否总成
        for (DynamicObject plm : collection) {

            String exchangetype = plm.getString("yem_exchangetype");
            String parentno;
            if ("D".equals(exchangetype)) {//新增
                parentno = plm.getString("yem_parent_now");
            } else {
                parentno = plm.getString("yem_textfield");
            }
            DynamicObject assatlasdet = seelctDyn("yem_em_assatlasdet", "number", parentno);//父项编码
            plm.set("yem_isassy", false);
            plm.set("yem_isresource", false);
            if (assatlasdet != null) {
                plm.set("yem_isassy", true);
            }
            logger.info("是否总成：{}，{}", parentno, assatlasdet != null);
            DynamicObject basemodelib = seelctDyn("yem_bd_basemodelib", "number", parentno);
            if (basemodelib != null) {
                plm.set("yem_isresource", true);
            }
            logger.info("是否资源：{}，{}", parentno, basemodelib != null);
        }
        // 替换关系生成成功之后更新完成时间、状态
        Map<Long, DynamicObject> successMap = new HashMap<>();
        DynamicObject bomChange = createBOMChange(plmObj, enabled);
        DynamicObjectCollection entry = bomChange.getDynamicObjectCollection("yem_entryentity");
        if (!entry.isEmpty()) {
            DynamicObject replace = null;
            String billno = plmObj.getString("billno");
            String gybg_no = plmObj.getString("yem_gybg_no");
            if (entry.size() == 1) {
                logger.info("生成替换关系：PLM单号：{}， PLM工艺变更单号：{}，父项编码：{}", billno, gybg_no, enabled.get(0).getString("yem_textfield"));
                logger.info("生成_当前替换关系：{}", entry.get(0).getString("yem_replace"));
                replace = AtlasUtils.Addreplace(bomChange, entry.get(0), null, "yem_em_replace", "add", "PLM");
                successMap.put(entry.get(0).getLong("id"), bomChange);
            }
            for (DynamicObject entryObject : entry) {
                int idx = entry.indexOf(entryObject);
                if (idx == 0) continue;
                logger.info("更新替换关系：PLM单号：{}， PLM工艺变更单号：{}，父项编码：{}", billno, gybg_no, entryObject.getString("yem_textfield"));
                logger.info("更新_替换关系：{}", entryObject.getString("yem_replace"));
                replace = AtlasUtils.Addreplace(bomChange, entry.get(0), replace, "yem_em_replace", "update", "PLM");
                successMap.put(entryObject.getLong("id"), bomChange);
            }
        }

        for (Map.Entry<Long, DynamicObject> mapEntry : successMap.entrySet()) {
            Long bomChangeEntryId = mapEntry.getKey();
            DynamicObject bomChangeObj = mapEntry.getValue();
            DynamicObjectCollection bomChangeEntry = bomChangeObj.getDynamicObjectCollection("yem_entryentity");
            int idx = 0;
            for (DynamicObject bom : bomChangeEntry) {
                if (bomChangeEntryId == bom.getLong("id")) {
                    idx = bomChangeEntry.indexOf(bom);
                }
            }
            DynamicObject currentEntry = bomChangeEntry.get(idx);
            Date completetime = currentEntry.getDate("yem_completetime");//完成时间
            for (DynamicObject d : collection) {
                long id = d.getLong("id");
                if (bomChangeEntryId == id) {
                    d.set("yem_invokstatus", "B");
                    d.set("yem_invoktime", completetime);
                }
            }
        }

        SaveServiceHelper.save(new DynamicObject[]{plmObj});
    }

    /**
     * 根据PLM创建BOM，用于生成替换关系
     *
     * @param head
     * @param entries
     * @return
     */
    private DynamicObject createBOMChange(DynamicObject head, DynamicObjectCollection entries) {
        DynamicObject bomchange = BusinessDataServiceHelper.newDynamicObject("yem_em_bomchange");
        bomchange.set("id", head.getLong("id"));
        bomchange.set("createorg", head.getDynamicObject("org"));//创建组织-组织
        bomchange.set("yem_producttype", head.getString("yem_producttype"));//产品分类
        bomchange.set("number", head.getString("billno"));//单据编号
        bomchange.set("yem_date", head.getDate("createtime"));//业务日期-创建时间
        bomchange.set("creator", head.getDynamicObject("creator"));//创建人
        DynamicObjectCollection detail = bomchange.getDynamicObjectCollection("yem_entryentity");
        for (DynamicObject entry : entries) {
            DynamicObject addNew = detail.addNew();
            addNew.set("id", entry.getLong("id"));

            String exchangetype = entry.getString("yem_exchangetype");

            if (exchangetype.equals("D")) {//新增PLM变更后对应BOM变更前
                addNew.set("yem_parentno", entry.getString("yem_parent_now"));//替换后-父件编码
                addNew.set("yem_parentname", entry.getString("yem_name_newparent"));//替换后-父项名称
                addNew.set("yem_subno", entry.getString("yem_child_now"));//替换后-子件编码
                addNew.set("yem_subname", entry.getString("yem_name_newchild"));//替换后-子件名称
                addNew.set("yem_subenname", "yem_englishname");//替换后-子件英文名称
                addNew.set("yem_qty", entry.getBigDecimal("yem_qty_new"));//替换后-数量
            } else {//否则变更前对应变更前
                addNew.set("yem_parentno", entry.getString("yem_textfield"));//替换前-父件编码
                addNew.set("yem_parentname", entry.getString("yem_name_parent"));//替换前-父项名称
                addNew.set("yem_subno", entry.getString("yem_child_before"));//替换前-子件编码
                addNew.set("yem_subname", entry.getString("yem_name_child"));//替换前-子件名称
                addNew.set("yem_subenname", "");//替换前-子件英文名称
                addNew.set("yem_qty", entry.getBigDecimal("yem_qty_before"));//替换前-数量
            }

            addNew.set("yem_enableparno", entry.getString("yem_parent_now"));//替换父件编码-父项编码
            addNew.set("yem_enableparname", entry.getString("yem_name_newparent"));//替换父件名称-父项名称
            addNew.set("yem_enablesubno", entry.getString("yem_child_now"));//替换子件编码-子项编码
            addNew.set("yem_enablesubname", entry.getString("yem_name_newchild"));//替换子件名称
            addNew.set("yem_enablesubenname", entry.getString("yem_englishname"));//替换件英文名称-子件英文名称
            addNew.set("yem_enablesubqty", entry.getBigDecimal("yem_qty_new"));//数量

            addNew.set("yem_enablesta", entry.getString("yem_enablesta"));//启用停用状态
            addNew.set("yem_replace", exchangetype);//替换关系
            addNew.set("yem_enableddate", entry.getDate("yem_starttime"));//启用时间
            addNew.set("yem_downdate", entry.getDate("yem_stoptime"));//停用时间
            addNew.set("yem_completetime", entry.getDate("yem_completetime"));//执行时间
            addNew.set("yem_remark", "");//备注
            addNew.set("yem_isass", entry.getBoolean("yem_isassy"));//是否总成
            addNew.set("yem_isresource", entry.getBoolean("yem_isresource"));//是否资源
        }
        return bomchange;
    }
}
